/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

/*
** $Date:: 2013-01-29 14:46:41 +0900 #$
** $Revision: 3722 $
** $Author: ossinfra@FITECHLABS.CO.JP $
**/

#include "common.h"
#include "log.h"
#include "db.h"

#include "jacommon.h"
#include "jastr.h"
#include "jatimeout.h"
#include "jastatus.h"
#include "jalog.h"
#include "../jarun/jaruniconjob.h"
#include "jajobiconjob.h"

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int jajob_icon_job_add_aborticon(zbx_uint64_t inner_job_id,
                                 zbx_uint64_t inner_job_id_fs_link)
{
    int db_ret;
    const char *__function_name = "jajob_icon_job_addabort";

    zabbix_log(LOG_LEVEL_DEBUG,
               "In %s() inner_job_id: " ZBX_FS_UI64
               ", inner_job_id_fs_link: " ZBX_FS_UI64, __function_name,
               inner_job_id, inner_job_id_fs_link);

    db_ret = ZBX_DB_OK;
    if (inner_job_id_fs_link == 0)
        goto exit;

    db_ret = DBexecute("insert into ja_run_job_table ("
                       "inner_job_id, inner_jobnet_id, inner_jobnet_main_id, invo_flag, job_type, status"
                       ") select " ZBX_FS_UI64
                       ", inner_jobnet_id, inner_jobnet_main_id, 1, %d, %d from ja_run_job_table"
                       " where inner_job_id = "
                       ZBX_FS_UI64, inner_job_id_fs_link,
                       JA_JOB_TYPE_ABORT, JA_JOB_STATUS_READY,
                       inner_job_id);
    if (db_ret <= ZBX_DB_OK)
        goto exit;

    db_ret =
        DBexecute("insert into ja_run_icon_job_table select " ZBX_FS_UI64
                  ", inner_jobnet_id, host_flag, stop_flag, command_type, NULL, hostname, NULL from ja_run_icon_job_table"
                  " where inner_job_id = " ZBX_FS_UI64,
                  inner_job_id_fs_link, inner_job_id);
    if (db_ret <= ZBX_DB_OK)
        goto exit;

    db_ret =
        DBexecute("insert into ja_run_job_command_table select "
                  ZBX_FS_UI64
                  ", inner_jobnet_id, command_cls, command from ja_run_job_command_table"
                  " where inner_job_id = " ZBX_FS_UI64,
                  inner_job_id_fs_link, inner_job_id);
    if (db_ret <= ZBX_DB_OK)
        goto exit;

    db_ret =
        DBexecute("insert into ja_run_value_job_table select " ZBX_FS_UI64
                  ", inner_jobnet_id, value_name, value from ja_run_value_job_table"
                  " where inner_job_id = " ZBX_FS_UI64,
                  inner_job_id_fs_link, inner_job_id);
    if (db_ret <= ZBX_DB_OK)
        goto exit;

    db_ret =
        DBexecute("insert into ja_run_value_jobcon_table select "
                  ZBX_FS_UI64
                  ", inner_jobnet_id, value_name, value from ja_run_value_jobcon_table"
                  " where inner_job_id = " ZBX_FS_UI64,
                  inner_job_id_fs_link, inner_job_id);
    if (db_ret <= ZBX_DB_OK)
        goto exit;

    db_ret =
        DBexecute("insert into ja_run_value_before_table select "
                  ZBX_FS_UI64
                  ", inner_jobnet_id, value_name, vefore_value from ja_run_value_before_table"
                  " where inner_job_id = " ZBX_FS_UI64,
                  inner_job_id_fs_link, inner_job_id);
    if (db_ret <= ZBX_DB_OK)
        goto exit;

  exit:
    if (db_ret == ZBX_DB_OK) {
        ja_log("JAJOBICONJOB200001", 0, NULL, __function_name,
               inner_job_id, inner_job_id_fs_link);
    }

    if (db_ret > ZBX_DB_OK) {
        return SUCCEED;
    } else {
        return FAIL;
    }
}

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int jajob_icon_job_abort(zbx_uint64_t inner_job_id)
{
    int ret;
    DB_RESULT result;
    DB_ROW row;
    const char *__function_name = "jajob_icon_job_abort";

    zabbix_log(LOG_LEVEL_DEBUG,
               "In %s() inner_job_id: " ZBX_FS_UI64, __function_name,
               inner_job_id);

    ret = FAIL;
    result =
        DBselect
        ("select stop_flag from ja_run_icon_job_table"
         " where inner_job_id = " ZBX_FS_UI64, inner_job_id);

    if (NULL != (row = DBfetch(result))) {
        if (atoi(row[0]) == 0) {
            ret = jarun_icon_job(inner_job_id, JA_AGENT_METHOD_KILL);
        } else {
            ret = jarun_icon_job(inner_job_id, JA_AGENT_METHOD_ABORT);
        }
    } else {
        ja_log("JAJOBICONJOB200002", 0, NULL, __function_name,
               inner_job_id);
        ret = ja_set_enderr(inner_job_id);
    }
    DBfree_result(result);
    return ret;
}

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int jajob_icon_job_timeout(zbx_uint64_t inner_job_id, char *start_time)
{
    int ret;
    zbx_uint64_t inner_jobnet_id;
    DB_RESULT result;
    DB_ROW row;
    const char *__function_name = "jajob_icon_job_timeout";

    zabbix_log(LOG_LEVEL_DEBUG, "In %s() inner_job_id: " ZBX_FS_UI64,
               __function_name, inner_job_id);

    ret = SUCCEED;
    result =
        DBselect
        ("select inner_jobnet_id, timeout from ja_run_icon_job_table"
         " where inner_job_id = " ZBX_FS_UI64, inner_job_id);
    if (NULL != (row = DBfetch(result))) {
        ZBX_STR2UINT64(inner_jobnet_id, row[0]);
        if (ja_timeout_check(row[1], start_time) == SUCCEED) {
            ja_log("JAJOBICONJOB300001", inner_jobnet_id, NULL,
                   __function_name, inner_job_id, row[1], start_time);

            if (ZBX_DB_OK >
                DBexecute
                ("update ja_run_job_table set timeout_flag = 1 where inner_job_id = "
                 ZBX_FS_UI64, inner_job_id))
                ret = FAIL;

            ja_joblog(JC_JOB_TIMEOUT, inner_jobnet_id, inner_job_id);
        }
    } else {
        ja_log("JAJOBICONJOB200002", 0, NULL, __function_name,
               inner_job_id);
        ret = ja_set_enderr(inner_job_id);
    }
    DBfree_result(result);
    return ret;
}
