/*
** Job Arranger for ZABBIX
** Copyright (C) 2012 FitechForce, Inc. All Rights Reserved.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/

/*
** $Date:: 2012-11-26 13:50:56 +0900 #$
** $Revision: 2730 $
** $Author: ossinfra@FITECHLABS.CO.JP $
**/

#include <json/json.h>
#include "common.h"
#include "comms.h"
#include "log.h"
#include "db.h"

#include "jacommon.h"
#include "jastatus.h"
#include "jaflow.h"
#include "javalue.h"
#include "jarunvalue.h"
#include "jajobobject.h"
#include "jaconnect.h"
#include "jatcp.h"
#include "jathreads.h"
#include "jaruniconfcopy.h"

/******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *                                                                            *
 * Purpose:                                                                   *
 *                                                                            *
 * Parameters:                                                                *
 *                                                                            *
 * Return value:                                                              *
 *                                                                            *
 * Comments:                                                                  *
 *                                                                            *
 ******************************************************************************/
int jarun_icon_fcopy(const zbx_uint64_t inner_job_id, const int method)
{
    int ret;
    DB_RESULT result;
    DB_ROW row;
    int from_host_flag, to_host_flag, overwrite_flag;
    char *from_host_name, *to_host_name;
    char *from_directory, *from_file_name, *to_directory;
    zbx_sock_t sock_from, sock_to;
    ja_job_object job, job_res;
    json_object *jp_arg;
    char buf[JA_MAX_DATA_LEN];
    ssize_t nbytes, written, mbytes;
    pid_t pid;
    const char *__function_name = "jarun_icon_fcopy";

    zabbix_log(LOG_LEVEL_DEBUG, "In %s() inner_job_id: " ZBX_FS_UI64,
               __function_name, inner_job_id);

    pid = ja_fork();
    if (pid == -1) {
        zabbix_log(LOG_LEVEL_ERR,
                   "In %s() can not fork(). inner_job_id: " ZBX_FS_UI64,
                   __function_name, inner_job_id);
        return ja_set_status_job(inner_job_id, JA_JOB_STATUS_RUNERR, -1,
                                 1);
    } else if (pid != 0) {
        waitpid(pid, NULL, WNOHANG);
        return SUCCEED;
    }

    DBconnect(ZBX_DB_CONNECT_ONCE);
    ret = FAIL;
    ja_job_object_init(&job);
    ja_job_object_init(&job_res);
    sock_from.socket = ZBX_SOCK_ERROR;
    sock_to.socket = ZBX_SOCK_ERROR;
    from_host_name = NULL;
    from_directory = NULL;
    from_file_name = NULL;
    to_host_name = NULL;
    to_directory = NULL;

    result =
        DBselect
        ("select * from ja_run_icon_fcopy_table where inner_job_id = "
         ZBX_FS_UI64, inner_job_id);
    if (NULL != (row = DBfetch(result))) {
        from_host_flag = atoi(row[2]);
        to_host_flag = atoi(row[3]);
        overwrite_flag = atoi(row[4]);
        from_host_name = zbx_strdup(from_host_name, row[5]);
        from_directory = zbx_strdup(from_directory, row[6]);
        from_file_name = zbx_strdup(from_file_name, row[7]);
        to_host_name = zbx_strdup(to_host_name, row[8]);
        to_directory = zbx_strdup(to_directory, row[9]);
    } else {
        DBfree_result(result);
        zabbix_log(LOG_LEVEL_ERR,
                   "In %s() can not find inner_job_id: " ZBX_FS_UI64,
                   __function_name, inner_job_id);
        ja_set_status_job(inner_job_id, JA_JOB_STATUS_ENDERR, -1, 1);
        exit(0);
    }
    DBfree_result(result);

    jp_arg = json_object_new_object();
    json_object_object_add(jp_arg, JA_PROTO_TAG_FROMDIR,
                           json_object_new_string(from_directory));
    json_object_object_add(jp_arg, JA_PROTO_TAG_FILENAME,
                           json_object_new_string(from_file_name));
    json_object_object_add(jp_arg, JA_PROTO_TAG_TODIR,
                           json_object_new_string(to_directory));
    json_object_object_add(jp_arg, JA_PROTO_TAG_OVERWRITE,
                           json_object_new_int(overwrite_flag));
    zbx_snprintf(job.argument, sizeof(job.argument), "%s",
                 json_object_to_json_string(jp_arg));
    json_object_put(jp_arg);

    zbx_snprintf(job.kind, sizeof(job.kind), "%s", JA_PROTO_VALUE_FCOPY);
    job.version = JA_PROTO_TELE_VERSION;
    job.jobid = inner_job_id;
    zbx_snprintf(job.serverid, sizeof(job.serverid), "%s", serverid);
    job.method = method;

    if (ja_connect(&sock_to, inner_job_id, to_host_flag, to_host_name) ==
        FAIL) {
        goto error;
    }
    zbx_snprintf(job.type, sizeof(job.type), "%s", JA_PROTO_VALUE_PUTFILE);
    if (ja_tcp_send_to(&sock_to, &job, CONFIG_TIMEOUT) == FAIL)
        goto error;
    if (ja_tcp_recv_to(&sock_to, &job_res, CONFIG_TIMEOUT) == FAIL)
        goto error;
    if (job_res.result != JA_RESPONSE_SUCCEED) {
        zabbix_log(LOG_LEVEL_ERR,
                   "In %s() inner_job_id: " ZBX_FS_UI64 " message: %s",
                   __function_name, inner_job_id, job_res.message);
        goto error;
    }
    if (ja_connect
        (&sock_from, inner_job_id, from_host_flag, from_host_name) == FAIL)
        goto error;
    zbx_snprintf(job.type, sizeof(job.type), "%s", JA_PROTO_VALUE_GETFILE);
    if (ja_tcp_send_to(&sock_from, &job, CONFIG_TIMEOUT) == FAIL)
        goto error;

    while (1) {
        ja_tcp_timeout_set(sock_from.socket, CONFIG_FCOPY_TIMEOUT);
        nbytes = ZBX_TCP_READ(sock_from.socket, (void *) buf, sizeof(buf));
        if (nbytes == ZBX_TCP_ERROR) {
            zabbix_log(LOG_LEVEL_ERR, "ZBX_TCP_READ() failed: %s",
                       strerror_from_system(zbx_sock_last_error()));
            goto error;
        }
        zabbix_log(LOG_LEVEL_DEBUG, "In %s() nbytes: %d", __function_name,
                   nbytes);
        if (nbytes == 0)
            break;

        mbytes = 0;
        written = 0;
        while (written < nbytes) {
            ja_tcp_timeout_set(sock_to.socket, CONFIG_FCOPY_TIMEOUT);
            mbytes =
                ZBX_TCP_WRITE(sock_to.socket, buf + written,
                              (int) (nbytes - written));
            if (mbytes == ZBX_TCP_ERROR) {
                zabbix_log(LOG_LEVEL_ERR, "ZBX_TCP_WRITE() failed: %s",
                           strerror_from_system(zbx_sock_last_error()));
                goto error;
            }
            written += mbytes;
        }
    }
    if (ja_tcp_recv_to(&sock_to, &job_res, CONFIG_FCOPY_TIMEOUT) == FAIL)
        goto error;
    if (job_res.result != JA_RESPONSE_SUCCEED) {
        zabbix_log(LOG_LEVEL_ERR,
                   "In %s() inner_job_id: " ZBX_FS_UI64 " message: %s",
                   __function_name, inner_job_id, job_res.message);
        goto error;
    }

    ret = SUCCEED;
  error:
    if (sock_from.socket >= 0)
        zbx_tcp_close(&sock_from);
    if (sock_to.socket >= 0)
        zbx_tcp_close(&sock_to);
    if (from_host_name != NULL)
        zbx_free(from_host_name);
    if (from_directory != NULL)
        zbx_free(from_directory);
    if (from_file_name != NULL)
        zbx_free(from_file_name);
    if (to_host_name != NULL)
        zbx_free(to_host_name);
    if (to_directory != NULL)
        zbx_free(to_directory);
    if (ret == SUCCEED) {
        jarun_set_value_after(inner_job_id);
        if (ja_flow(inner_job_id, JA_FLOW_TYPE_NORMAL) == FAIL)
            ja_set_enderr(inner_job_id);
    } else {
        ja_set_status_job(inner_job_id, JA_JOB_STATUS_RUNERR, -1, 1);
    }
    DBclose();
    zabbix_log(LOG_LEVEL_DEBUG, "In %s() END", __function_name);
    exit(0);
}
